import 'dart:ui' as ui;

import 'package:dgg_flutter_toast/src/core/toast_manager.dart';
import 'package:dgg_flutter_toast/src/widget/toast_theme.dart';
import 'package:flutter/material.dart';

import 'core/toast_position.dart';

const _defaultDuration = Duration(
  milliseconds: 2300,
);

/// 显示单个toast
void showToast(
  BuildContext context,
  String msg, {
  Color backgroundColor,
  double radius,
  TextStyle textStyle,
  TextAlign textAlign,
  EdgeInsetsGeometry textPadding,
  ToastPosition position,
  Duration duration,
  bool dismissOtherOnShow,
}) {
  if (!(msg is String)) {
    msg = '';
  }

  backgroundColor ??=
      ToastTheme.of(context).backgroundColor ?? const Color(0xDD000000);

  radius ??= ToastTheme.of(context).radius ?? 10.0;

  textStyle ??= ToastTheme.of(context).textStyle ??
      const TextStyle(
        decoration: TextDecoration.none,
        fontSize: 15.0,
        fontWeight: FontWeight.normal,
        color: Colors.white,
      );

  textAlign ??= ToastTheme.of(context).textAlign ?? TextAlign.center;

  textPadding ??= ToastTheme.of(context).textPadding ??
      const EdgeInsets.symmetric(
        horizontal: 8.0,
        vertical: 4.0,
      );

  position ??= ToastTheme.of(context).position ?? ToastPosition.center;
  EdgeInsets padding = EdgeInsets.zero;
  //位置偏移量计算
  if (position.offset > 0) { 
    if (position.align == Alignment.center || position.align == Alignment.centerLeft || position.align == Alignment.centerRight) {
      padding = EdgeInsets.only(top: position.offset*2);
    }else {
      padding = EdgeInsets.only(top: position.offset);
    }
  }else if (position.offset < 0) {
    if (position.align == Alignment.center || position.align == Alignment.centerLeft || position.align == Alignment.centerRight) {
      padding = EdgeInsets.only(bottom: position.offset.abs()*2);
    }else {
      padding = EdgeInsets.only(bottom: position.offset.abs());
    }
  }else {
    padding = padding;
  }

  duration ??= ToastTheme.of(context).duration ?? _defaultDuration;

  dismissOtherOnShow ??= ToastTheme.of(context).dismissOtherOnShow ?? false;

  Widget widget = Container(
    alignment: position.align,
    padding: padding,
    child: UnconstrainedBox(
      child: Container(
        padding: textPadding,
        margin: EdgeInsets.all(25),
        decoration: BoxDecoration(
          color: backgroundColor,
          borderRadius: BorderRadius.circular(radius),
        ),
        child: Center(
          child: Text(
            msg,
            style: textStyle,
            textAlign: textAlign,
          ),
        ),
      ),
    ),
  );

  OverlayEntry entry = OverlayEntry(
    builder: (_) => widget,
    opaque: false,
    maintainState: false,
  );

  if (dismissOtherOnShow) {
    ToastManager().dismissAll();
  }

  Overlay.of(context).insert(entry);
  ToastManager().addToastEntry(entry);

  Future.delayed(duration, () {
    ToastManager().removeToastEntry(entry);
  });
}

///关闭所有toast
void dismissAllToast() {
  ToastManager().dismissAll();
}
