
import 'package:dgg_flutter_toast/src/core/toast_position.dart';
import 'package:dgg_flutter_toast/src/widget/toast_theme.dart';
import 'package:flutter/material.dart';

class DGGToastThemeWidget extends StatefulWidget {
  final Widget child; 

  ///背景颜色
  final Color backgroundColor;

  ///圆角
  final double radius;

  ///文本样式
  final TextStyle textStyle;

  ///文本对齐方式
  final TextAlign textAlign;

  ///文本与背景框的间距
  final EdgeInsets textPadding;

  ///弹出位置
  final ToastPosition position;

  ///显示时长
  final Duration duration;

  ///弹出新toast时，是否关闭其他已经显示的toast
  final bool dismissOtherOnShow;

  const DGGToastThemeWidget({
    Key key,
    @required this.child,
    Color backgroundColor,
    this.radius = 10.0,
    this.textStyle,
    this.textAlign,
    this.textPadding,
    this.position = ToastPosition.center,
    this.duration, 
    this.dismissOtherOnShow = false,
  })  : this.backgroundColor = backgroundColor ?? const Color(0xDD000000),
        super(key: key);

  @override
  _DGGToastThemeWidgetState createState() => _DGGToastThemeWidgetState();
}

class _DGGToastThemeWidgetState extends State<DGGToastThemeWidget> {
  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    var overlay = Overlay(
      initialEntries: [
        OverlayEntry(
          builder: (ctx) {
            return widget.child;
          },
        ),
      ],
    );

    Widget w = Directionality(
      child: Stack(children: <Widget>[
        overlay,
        Positioned(
          left: 0.0,
          right: 0.0,
          top: 0.0,
          bottom: 0.0,
          child: IgnorePointer(
            child: Container(
              color: Colors.black.withOpacity(0.0),
            ),
          ),
        )
      ]),
      textDirection: TextDirection.ltr,
    );

    var typography = Typography(platform: TargetPlatform.android);
    final TextTheme defaultTextTheme = typography.white;

    TextStyle textStyle = widget.textStyle ??
        defaultTextTheme.body1.copyWith(
          fontSize: 15.0,
          fontWeight: FontWeight.normal,
          color: Colors.white,
        );

    TextAlign textAlign = widget.textAlign ?? TextAlign.center;
    
    EdgeInsets textPadding = widget.textPadding ??
        const EdgeInsets.symmetric(
          horizontal: 8.0,
          vertical: 4.0,
        );

    return ToastTheme(
      child: w,
      backgroundColor: widget.backgroundColor,
      radius: widget.radius,
      textStyle: textStyle,
      position: widget.position,
      dismissOtherOnShow: widget.dismissOtherOnShow,
      textAlign: textAlign,
      textPadding: textPadding,
      duration: widget.duration,
    );
  }
}